"use strict";

var _embarkI18n = require("embark-i18n");

require('colors');

let async = require('async');

class Swarm {
  constructor(options) {
    this.options = options;
    this.buildDir = options.buildDir || 'dist/';
    this.swarm = options.swarm;
    this.providerUrl = options.providerUrl;
    this.env = options.env;
  }

  deploy(cb) {
    console.log((0, _embarkI18n.__)("deploying to swarm!"));
    const self = this;
    const swarm = this.swarm;
    async.waterfall([function runCommand(callback) {
      console.log(("=== " + (0, _embarkI18n.__)("adding %s to swarm", self.buildDir)).green);
      swarm.uploadDirectory(self.buildDir, 'index.html', callback);
    }, function printUrls(dir_hash, callback) {
      if (!dir_hash) {
        return callback('No directory hash was returned');
      }

      console.log(("=== " + (0, _embarkI18n.__)("DApp available at") + ` ${self.providerUrl}/bzz:/${dir_hash}/index.html`).green);
      console.log(("=== " + (0, _embarkI18n.__)("DApp available at") + ` https://swarm-gateways.net/bzz:/${dir_hash}/index.html`).green);

      if (self.env === 'development') {
        console.log(("\n=== " + "Blockchain must be running".bold + " ===").yellow);
        console.log(("embark run".italic + (0, _embarkI18n.__)(" or ") + "embark blockchain".italic + (0, _embarkI18n.__)(" must be running when the site is loaded to interact with the blockchain.\n")).yellow);
        console.log(("=== " + "Usage with the public gateway".bold + " ===").yellow);
        console.log(((0, _embarkI18n.__)("If you wish to load your development site from the public gateway (swarm-gateways.net), you will need to first update your CORS settings (") + "config/blockchain.js > wsOrigins".italic + (0, _embarkI18n.__)(" and ") + "config/blockchain.js > rpcCorsDomain".italic + (0, _embarkI18n.__)(") to allow ") + "swarm-gateways.net".underline + (0, _embarkI18n.__)(". If these were set to 'auto', they would now need to be set to ") + "https://swarm-gateways.net,http://localhost:8000,http://localhost:8500,embark".underline + ".\n").yellow);
      }

      callback(null, dir_hash);
    }], function (err, dir_hash) {
      if (err) {
        console.log((0, _embarkI18n.__)("error uploading to swarm").red);
        console.log(err);
        return cb(err);
      }

      cb(null, dir_hash);
    });
  }

}

module.exports = Swarm;
//# sourceMappingURL=upload.js.map